==> json-joy-6.1.0.tgz: package/esm/json-pack/Decoder/v9.js <==
import { JsonPackValue } from '..';
import { JsonPackExtension } from '../JsonPackExtension';
import { CachedKeyDecoder } from './CachedKeyDecoder';
const sharedCachedKeyDecoder = new CachedKeyDecoder();
export class Decoder {
    keyDecoder;
    uint8 = new Uint8Array([]);
    view = new DataView(this.uint8.buffer);
    x = 0;
    constructor(keyDecoder = sharedCachedKeyDecoder) {
        this.keyDecoder = keyDecoder;
    }
    reset(uint8) {
        this.x = 0;
        this.uint8 = uint8;
        this.view = new DataView(uint8.buffer, uint8.byteOffset, uint8.byteLength);
    }
    decode(uint8) {
        this.reset(uint8);
        return this.val();
    }
    val() {
        const byte = this.u8();
        if (byte >= 0xe0)
            return byte - 0x100;
        if (byte <= 0xbf) {
            if (byte < 0x90) {
                if (byte <= 0b1111111)
                    return byte;
                return this.obj(byte & 0b1111);
            }
            else {
                if (byte < 0xa0)
                    return this.arr(byte & 0b1111);
                else
                    return this.str(byte & 0b11111);
            }
        }
        if (byte <= 0xce) {
            if (byte <= 0xc8) {
                if (byte <= 0xc4) {
                    if (byte <= 0xc2)
                        return byte === 0xc2 ? false : null;
                    else
                        return byte === 0xc4 ? this.bin(this.u8()) : true;
                }
                else {
                    if (byte <= 0xc6)
                        return byte === 0xc6 ? this.bin(this.u32()) : this.bin(this.u16());
                    else
                        return byte === 0xc8 ? this.ext(this.u16()) : this.ext(this.u8());
                }
            }
            else {
                return byte <= 0xcc
                    ? byte <= 0xca
                        ? byte === 0xca
                            ? this.f32()
                            : this.ext(this.u32())
                        : byte === 0xcc
                            ? this.u8()
                            : this.f64()
                    : byte <= 0xce
                        ? byte === 0xce
                            ? this.u32()
                            : this.u16()
                        : byte === 0xd0
                            ? this.i8()
                            : this.u32() * 4294967296 + this.u32();
            }
        }
        else if (byte <= 0xd8) {
            return byte <= 0xd4
                ? byte <= 0xd2
                    ? byte === 0xd2
                        ? this.i32()
                        : this.i16()
                    : byte === 0xd4
                        ? this.ext(1)
                        : this.i32() * 4294967296 + this.i32()
                : byte <= 0xd6
                    ? byte === 0xd6
                        ? this.ext(4)
                        : this.ext(2)
                    : byte === 0xd8
                        ? this.ext(16)
                        : this.ext(8);
        }
        else {
            switch (byte) {
                case 0xd9:
                    return this.str(this.u8());
                case 0xda:
                    return this.str(this.u16());
                case 0xdb:
                    return this.str(this.u32());
                case 0xdc:
                    return this.arr(this.u16());
                case 0xdd:
                    return this.arr(this.u32());
                case 0xde:
                    return this.obj(this.u16());
                case 0xdf:
                    return this.obj(this.u32());
            }
        }
        return undefined;
    }
    str(size) {
        const uint8 = this.uint8;
        const end = this.x + size;
        let x = this.x;
        let str = '';
        while (x < end) {
            const b1 = uint8[x++];
            if ((b1 & 0x80) === 0) {
                str += String.fromCharCode(b1);
                continue;
            }
            else if ((b1 & 0xe0) === 0xc0) {
                str += String.fromCharCode(((b1 & 0x1f) << 6) | (uint8[x++] & 0x3f));
            }
            else if ((b1 & 0xf0) === 0xe0) {
                str += String.fromCharCode(((b1 & 0x1f) << 12) | ((uint8[x++] & 0x3f) << 6) | (uint8[x++] & 0x3f));
            }
            else if ((b1 & 0xf8) === 0xf0) {
                const b2 = uint8[x++] & 0x3f;
                const b3 = uint8[x++] & 0x3f;
                const b4 = uint8[x++] & 0x3f;
                let code = ((b1 & 0x07) << 0x12) | (b2 << 0x0c) | (b3 << 0x06) | b4;
                if (code > 0xffff) {
                    code -= 0x10000;
                    str += String.fromCharCode(((code >>> 10) & 0x3ff) | 0xd800);
                    code = 0xdc00 | (code & 0x3ff);
                }
                str += String.fromCharCode(code);
            }
            else {
                str += String.fromCharCode(b1);
            }
        }
        this.x = end;
        return str;
    }
    obj(size) {
        const obj = {};
        for (let i = 0; i < size; i++) {
            const key = this.key();
            obj[key] = this.val();
        }
        return obj;
    }
    key() {
        const byte = this.view.getUint8(this.x);
        if (byte >= 0b10100000 && byte <= 0b10111111) {
            const size = byte & 0b11111;
            const key = this.keyDecoder.decode(this.uint8, this.x + 1, size);
            this.x += 1 + size;
            return key;
        }
        else if (byte === 0xd9) {
            const size = this.view.getUint8(this.x + 1);
            if (size < 32) {
                const key = this.keyDecoder.decode(this.uint8, this.x + 2, size);
                this.x += 2 + size;
                return key;
            }
            else
                return this.val();
        }
        else
            return this.val();
    }
    arr(size) {
        const arr = [];
        for (let i = 0; i < size; i++)
            arr.push(this.val());
        return arr;
    }
    bin(size) {
        const end = this.x + size;
        const bin = this.uint8.subarray(this.x, end);
        this.x = end;
        return bin;
    }
    ext(size) {
        const type = this.u8();
        const end = this.x + size;
        const buf = this.uint8.subarray(this.x, end);
        this.x = end;
        return new JsonPackExtension(type, buf);
    }
    back(bytes) {
        this.x -= bytes;
    }
    u8() {
        return this.view.getUint8(this.x++);
    }
    u16() {
        const num = this.view.getUint16(this.x);
        this.x += 2;
        return num;
    }
    u32() {
        const num = this.view.getUint32(this.x);
        this.x += 4;
        return num;
    }
    i8() {
        return this.view.getInt8(this.x++);
    }
    i16() {
        const num = this.view.getInt16(this.x);
        this.x += 2;
        return num;
    }
    i32() {
        const num = this.view.getInt32(this.x);
        this.x += 4;
        return num;
    }
    f32() {
        const pos = this.x;
        this.x += 4;
        return this.view.getFloat32(pos);
    }
    f64() {
        const pos = this.x;
        this.x += 8;
        return this.view.getFloat64(pos);
    }
    decodeOneLevel(uint8) {
        this.reset(uint8);
        return this.valOneLevel();
    }
    valOneLevel() {
        const byte = this.view.getUint8(this.x);
        const isMap = byte === 0xde || byte === 0xdf || (byte >> 4 === 0b1000);
        if (isMap) {
            this.x++;
            const size = byte === 0xde ? this.u16() : byte === 0xdf ? this.u32() : (byte & 0b1111);
            const obj = {};
            for (let i = 0; i < size; i++) {
                const key = this.key();
                obj[key] = this.primitive();
            }
            return obj;
        }
        const isArray = byte === 0xdc || byte === 0xdd || (byte >> 4 === 0b1001);
        if (isArray) {
            this.x++;
            const size = byte === 0xdc ? this.u16() : byte === 0xdd ? this.u32() : (byte & 0b1111);
            const arr = [];
            for (let i = 0; i < size; i++)
                arr.push(this.primitive());
            return arr;
        }
        return this.val();
    }
    primitive() {
        const byte = this.view.getUint8(this.x);
        const isMapOrArray = byte === 0xde || byte === 0xdf || byte === 0xdc || byte === 0xdd || (byte >> 5 === 0b100);
        if (isMapOrArray) {
            const length = this.valSkip();
            this.x -= length;
            const buf = this.bin(length);
            return new JsonPackValue(buf);
        }
        return this.val();
    }
    skip(length) {
        this.x += length;
        return length;
    }
    valSkip() {
        const byte = this.u8();
        if (byte >= 0xe0)
            return 1;
        if (byte <= 0xbf) {
            if (byte < 0x90) {
                if (byte <= 0b1111111)
                    return 1;
                return 1 + this.objSkip(byte & 0b1111);
            }
            else {
                if (byte < 0xa0)
                    return 1 + this.arrSkip(byte & 0b1111);
                else
                    return 1 + this.skip(byte & 0b11111);
            }
        }
        if (byte <= 0xce) {
            if (byte <= 0xc8) {
                if (byte <= 0xc4) {
                    if (byte <= 0xc2)
                        return byte === 0xc2 ? 1 : 1;
                    else
                        return byte === 0xc4 ? 2 + this.skip(this.u8()) : 1;
                }
                else {
                    if (byte <= 0xc6)
                        return byte === 0xc6 ? 5 + this.skip(this.u32()) : 3 + this.skip(this.u16());
                    else
                        return byte === 0xc8 ? 4 + this.skip(this.u16()) : 3 + this.skip(this.u8());
                }
            }
            else {
                return byte <= 0xcc
                    ? byte <= 0xca
                        ? byte === 0xca
                            ? 1 + this.skip(4)
                            : 1 + 1 + 4 + this.skip(this.u32())
                        : byte === 0xcc
                            ? 1 + this.skip(1)
                            : 1 + this.skip(8)
                    : byte <= 0xce
                        ? byte === 0xce
                            ? 1 + this.skip(4)
                            : 1 + this.skip(2)
                        : byte === 0xd0
                            ? 1 + this.skip(1)
                            : 1 + this.skip(8);
            }
        }
        else if (byte <= 0xd8) {
            return byte <= 0xd4
                ? byte <= 0xd2
                    ? byte === 0xd2
                        ? 1 + this.skip(4)
                        : 1 + this.skip(2)
                    : byte === 0xd4
                        ? 1 + this.skip(2)
                        : 1 + this.skip(8)
                : byte <= 0xd6
                    ? byte === 0xd6
                        ? 1 + this.skip(5)
                        : 1 + this.skip(3)
                    : byte === 0xd8
                        ? 1 + this.skip(17)
                        : 1 + this.skip(9);
        }
        else {
            switch (byte) {
                case 0xd9:
                    return 2 + this.skip(this.u8());
                case 0xda:
                    return 3 + this.skip(this.u16());
                case 0xdb:
                    return 5 + this.skip(this.u32());
                case 0xdc:
                    return 3 + this.arrSkip(this.u16());
                case 0xdd:
                    return 5 + this.arrSkip(this.u32());
                case 0xde:
                    return 3 + this.objSkip(this.u16());
                case 0xdf:
                    return 5 + this.objSkip(this.u32());
            }
        }
        return 1;
    }
    arrSkip(size) {
        let length = 0;
        for (let i = 0; i < size; i++)
            length += this.valSkip();
        return length;
    }
    objSkip(size) {
        let length = 0;
        for (let i = 0; i < size; i++) {
            length += this.valSkip() + this.valSkip();
        }
        return length;
    }
}

==> json-joy-7.0.0.tgz: package/esm/json-pack/Decoder/v9.js <==
import { JsonPackValue } from '..';
import { JsonPackExtension } from '../JsonPackExtension';
import { CachedKeyDecoder } from './CachedKeyDecoder';
const sharedCachedKeyDecoder = new CachedKeyDecoder();
export class Decoder {
    keyDecoder;
    uint8 = new Uint8Array([]);
    view = new DataView(this.uint8.buffer);
    x = 0;
    constructor(keyDecoder = sharedCachedKeyDecoder) {
        this.keyDecoder = keyDecoder;
    }
    reset(uint8) {
        this.x = 0;
        this.uint8 = uint8;
        this.view = new DataView(uint8.buffer, uint8.byteOffset, uint8.byteLength);
    }
    decode(uint8) {
        this.reset(uint8);
        return this.val();
    }
    val() {
        const byte = this.u8();
        if (byte >= 0xe0)
            return byte - 0x100;
        if (byte <= 0xbf) {
            if (byte < 0x90) {
                if (byte <= 0b1111111)
                    return byte;
                return this.obj(byte & 0b1111);
            }
            else {
                if (byte < 0xa0)
                    return this.arr(byte & 0b1111);
                else
                    return this.str(byte & 0b11111);
            }
        }
        if (byte <= 0xce) {
            if (byte <= 0xc8) {
                if (byte <= 0xc4) {
                    if (byte <= 0xc2)
                        return byte === 0xc2 ? false : null;
                    else
                        return byte === 0xc4 ? this.bin(this.u8()) : true;
                }
                else {
                    if (byte <= 0xc6)
                        return byte === 0xc6 ? this.bin(this.u32()) : this.bin(this.u16());
                    else
                        return byte === 0xc8 ? this.ext(this.u16()) : this.ext(this.u8());
                }
            }
            else {
                return byte <= 0xcc
                    ? byte <= 0xca
                        ? byte === 0xca
                            ? this.f32()
                            : this.ext(this.u32())
                        : byte === 0xcc
                            ? this.u8()
                            : this.f64()
                    : byte <= 0xce
                        ? byte === 0xce
                            ? this.u32()
                            : this.u16()
                        : byte === 0xd0
                            ? this.i8()
                            : this.u32() * 4294967296 + this.u32();
            }
        }
        else if (byte <= 0xd8) {
            return byte <= 0xd4
                ? byte <= 0xd2
                    ? byte === 0xd2
                        ? this.i32()
                        : this.i16()
                    : byte === 0xd4
                        ? this.ext(1)
                        : this.i32() * 4294967296 + this.i32()
                : byte <= 0xd6
                    ? byte === 0xd6
                        ? this.ext(4)
                        : this.ext(2)
                    : byte === 0xd8
                        ? this.ext(16)
                        : this.ext(8);
        }
        else {
            switch (byte) {
                case 0xd9:
                    return this.str(this.u8());
                case 0xda:
                    return this.str(this.u16());
                case 0xdb:
                    return this.str(this.u32());
                case 0xdc:
                    return this.arr(this.u16());
                case 0xdd:
                    return this.arr(this.u32());
                case 0xde:
                    return this.obj(this.u16());
                case 0xdf:
                    return this.obj(this.u32());
            }
        }
        return undefined;
    }
    str(size) {
        const uint8 = this.uint8;
        const end = this.x + size;
        let x = this.x;
        let str = '';
        while (x < end) {
            const b1 = uint8[x++];
            if ((b1 & 0x80) === 0) {
                str += String.fromCharCode(b1);
                continue;
            }
            else if ((b1 & 0xe0) === 0xc0) {
                str += String.fromCharCode(((b1 & 0x1f) << 6) | (uint8[x++] & 0x3f));
            }
            else if ((b1 & 0xf0) === 0xe0) {
                str += String.fromCharCode(((b1 & 0x1f) << 12) | ((uint8[x++] & 0x3f) << 6) | (uint8[x++] & 0x3f));
            }
            else if ((b1 & 0xf8) === 0xf0) {
                const b2 = uint8[x++] & 0x3f;
                const b3 = uint8[x++] & 0x3f;
                const b4 = uint8[x++] & 0x3f;
                let code = ((b1 & 0x07) << 0x12) | (b2 << 0x0c) | (b3 << 0x06) | b4;
                if (code > 0xffff) {
                    code -= 0x10000;
                    str += String.fromCharCode(((code >>> 10) & 0x3ff) | 0xd800);
                    code = 0xdc00 | (code & 0x3ff);
                }
                str += String.fromCharCode(code);
            }
            else {
                str += String.fromCharCode(b1);
            }
        }
        this.x = end;
        return str;
    }
    obj(size) {
        const obj = {};
        for (let i = 0; i < size; i++) {
            const key = this.key();
            obj[key] = this.val();
        }
        return obj;
    }
    key() {
        const byte = this.view.getUint8(this.x);
        if (byte >= 0b10100000 && byte <= 0b10111111) {
            const size = byte & 0b11111;
            const key = this.keyDecoder.decode(this.uint8, this.x + 1, size);
            this.x += 1 + size;
            return key;
        }
        else if (byte === 0xd9) {
            const size = this.view.getUint8(this.x + 1);
            if (size < 32) {
                const key = this.keyDecoder.decode(this.uint8, this.x + 2, size);
                this.x += 2 + size;
                return key;
            }
            else
                return this.val();
        }
        else
            return this.val();
    }
    arr(size) {
        const arr = [];
        for (let i = 0; i < size; i++)
            arr.push(this.val());
        return arr;
    }
    bin(size) {
        const end = this.x + size;
        const bin = this.uint8.subarray(this.x, end);
        this.x = end;
        return bin;
    }
    ext(size) {
        const type = this.u8();
        const end = this.x + size;
        const buf = this.uint8.subarray(this.x, end);
        this.x = end;
        return new JsonPackExtension(type, buf);
    }
    back(bytes) {
        this.x -= bytes;
    }
    u8() {
        return this.view.getUint8(this.x++);
    }
    u16() {
        const num = this.view.getUint16(this.x);
        this.x += 2;
        return num;
    }
    u32() {
        const num = this.view.getUint32(this.x);
        this.x += 4;
        return num;
    }
    i8() {
        return this.view.getInt8(this.x++);
    }
    i16() {
        const num = this.view.getInt16(this.x);
        this.x += 2;
        return num;
    }
    i32() {
        const num = this.view.getInt32(this.x);
        this.x += 4;
        return num;
    }
    f32() {
        const pos = this.x;
        this.x += 4;
        return this.view.getFloat32(pos);
    }
    f64() {
        const pos = this.x;
        this.x += 8;
        return this.view.getFloat64(pos);
    }
    decodeOneLevel(uint8) {
        this.reset(uint8);
        return this.valOneLevel();
    }
    valOneLevel() {
        const byte = this.view.getUint8(this.x);
        const isMap = byte === 0xde || byte === 0xdf || byte >> 4 === 0b1000;
        if (isMap) {
            this.x++;
            const size = byte === 0xde ? this.u16() : byte === 0xdf ? this.u32() : byte & 0b1111;
            const obj = {};
            for (let i = 0; i < size; i++) {
                const key = this.key();
                obj[key] = this.primitive();
            }
            return obj;
        }
        const isArray = byte === 0xdc || byte === 0xdd || byte >> 4 === 0b1001;
        if (isArray) {
            this.x++;
            const size = byte === 0xdc ? this.u16() : byte === 0xdd ? this.u32() : byte & 0b1111;
            const arr = [];
            for (let i = 0; i < size; i++)
                arr.push(this.primitive());
            return arr;
        }
        return this.val();
    }
    primitive() {
        const byte = this.view.getUint8(this.x);
        const isMapOrArray = byte === 0xde || byte === 0xdf || byte === 0xdc || byte === 0xdd || byte >> 5 === 0b100;
        if (isMapOrArray) {
            const length = this.valSkip();
            this.x -= length;
            const buf = this.bin(length);
            return new JsonPackValue(buf);
        }
        return this.val();
    }
    skip(length) {
        this.x += length;
        return length;
    }
    valSkip() {
        const byte = this.u8();
        if (byte >= 0xe0)
            return 1;
        if (byte <= 0xbf) {
            if (byte < 0x90) {
                if (byte <= 0b1111111)
                    return 1;
                return 1 + this.objSkip(byte & 0b1111);
            }
            else {
                if (byte < 0xa0)
                    return 1 + this.arrSkip(byte & 0b1111);
                else
                    return 1 + this.skip(byte & 0b11111);
            }
        }
        if (byte <= 0xce) {
            if (byte <= 0xc8) {
                if (byte <= 0xc4) {
                    if (byte <= 0xc2)
                        return byte === 0xc2 ? 1 : 1;
                    else
                        return byte === 0xc4 ? 2 + this.skip(this.u8()) : 1;
                }
                else {
                    if (byte <= 0xc6)
                        return byte === 0xc6 ? 5 + this.skip(this.u32()) : 3 + this.skip(this.u16());
                    else
                        return byte === 0xc8 ? 4 + this.skip(this.u16()) : 3 + this.skip(this.u8());
                }
            }
            else {
                return byte <= 0xcc
                    ? byte <= 0xca
                        ? byte === 0xca
                            ? 1 + this.skip(4)
                            : 1 + 1 + 4 + this.skip(this.u32())
                        : byte === 0xcc
                            ? 1 + this.skip(1)
                            : 1 + this.skip(8)
                    : byte <= 0xce
                        ? byte === 0xce
                            ? 1 + this.skip(4)
                            : 1 + this.skip(2)
                        : byte === 0xd0
                            ? 1 + this.skip(1)
                            : 1 + this.skip(8);
            }
        }
        else if (byte <= 0xd8) {
            return byte <= 0xd4
                ? byte <= 0xd2
                    ? byte === 0xd2
                        ? 1 + this.skip(4)
                        : 1 + this.skip(2)
                    : byte === 0xd4
                        ? 1 + this.skip(2)
                        : 1 + this.skip(8)
                : byte <= 0xd6
                    ? byte === 0xd6
                        ? 1 + this.skip(5)
                        : 1 + this.skip(3)
                    : byte === 0xd8
                        ? 1 + this.skip(17)
                        : 1 + this.skip(9);
        }
        else {
            switch (byte) {
                case 0xd9:
                    return 2 + this.skip(this.u8());
                case 0xda:
                    return 3 + this.skip(this.u16());
                case 0xdb:
                    return 5 + this.skip(this.u32());
                case 0xdc:
                    return 3 + this.arrSkip(this.u16());
                case 0xdd:
                    return 5 + this.arrSkip(this.u32());
                case 0xde:
                    return 3 + this.objSkip(this.u16());
                case 0xdf:
                    return 5 + this.objSkip(this.u32());
            }
        }
        return 1;
    }
    arrSkip(size) {
        let length = 0;
        for (let i = 0; i < size; i++)
            length += this.valSkip();
        return length;
    }
    objSkip(size) {
        let length = 0;
        for (let i = 0; i < size; i++) {
            length += this.valSkip() + this.valSkip();
        }
        return length;
    }
}
